-- main.lua
function love.load(arg)
  -- declare shorthand framework names
  lg = love.graphics
  li = love.image
  la = love.audio
  lm = love.mouse
  lk = love.keyboard
  lt = love.timer
  le = love.event
  ls = love.system
  lw = love.window
  lf = love.filesystem

  lg.setDefaultFilter('nearest', 'nearest')

  -- import third-party libraries
  require 'lib.autobatch'
  require 'lib.tesound'
  Gamestate = require 'lib.hump.gamestate'
  Class = require 'lib.hump.class'
  Signal = require 'lib.hump.signal'
  Timer = require 'lib.hump.timer'
  Vector = require 'lib.hump.vector'
  Camera = require 'lib.hump.camera'
  Bump = require 'lib.bump'
  Anim8 = require 'lib.anim8'

  -- load setup
  local settings = require 'setup.settings'
  require 'setup.fonts'

  -- load modules
  -- require 'modules.foo'

  -- set up states
  require 'states.title'
  require 'states.game'
  require 'states.instruction'
  require 'states.victory'

  SPRITESHEET = lg.newImage('images/snake-roy-spritesheet.png')

  -- TODO later user settings?

  -- window/scaling setup:

  TINYCANVAS = lg.newCanvas(settings.pxWidth, settings.pxHeight)

  local w, h, flags = lw.getMode()
  lw.setMode(
    settings.pxWidth * settings.scale,
    settings.pxHeight * settings.scale,
    flags
  )
  Gflags = flags

  w, h, flags = lw.getMode()
  GscaleX, GscaleY = w/settings.pxWidth, h/settings.pxHeight
  -- custom gamestate transition fader, via https://github.com/vrld/moldslife/blob/master/main.lua#L41
    function Gamestate.transition(to, duration, ...)
    	duration = duration or 1

    	local fade_color, t = {24, 20, 37, 0}, 0
    	local draw, switch, transition = Gamestate.draw, Gamestate.switch, Gamestate.transition
    	Gamestate.draw = function()
    		draw()
        local w, h = love.graphics.getDimensions()
    		color = {love.graphics.getColor()}
    		love.graphics.setColor(fade_color)
    		love.graphics.rectangle('fill', 0,0, w, h)
    		love.graphics.setColor(color)
    	end
    	-- disable switching states while in transition
    	Gamestate.switch = function() end
    	Gamestate.transition = function() end

    	local args = {...}
    	Timer.script(function(wait)
    		Timer.tween(duration/2, fade_color, {[4] = 255}, 'linear')
    		wait(duration / 2)
    		switch(to, unpack(args))
    		Timer.tween(duration/2, fade_color, {[4] = 0}, 'linear')
    		wait(duration / 2)
    		Gamestate.draw, Gamestate.switch, Gamestate.transition = draw, switch, transition
    	end)
    end

  lg.setBackgroundColor({0,128,9})
  Gamestate.registerEvents()
  Gamestate.switch(TitleScreen)

end

function love.update(dt)
  if not Game.paused then Timer.update(dt) end
  TEsound.cleanup()
end

function love.draw()

end

function love.keypressed(key)
  --local settings = require 'modules.setup.settings'
  if key == 'f4' and (lk.isDown('ralt') or lk.isDown('lalt')) then
    le.quit()
  elseif key == 'p' then
    local screenshot = lg.newScreenshot()
    screenshot:encode('png', os.time() .. '.png')
  elseif key == 'f1' or key == 'f11' then

    -- FIXME

    -- local fullscreen, fstype = lw.getFullscreen()
    -- local flags = Gflags
    -- if fullscreen then
    --   local w, h = settings.pxWidth * settings.scale, settings.pxHeight * settings.scale
    --   flags.fullscreen = false
    --   flags.borderless = false
    --   lw.setMode(w, h, flags)
    --   w, h, flags = lw.getMode()
    --   GscaleX, GscaleY = w/settings.pxWidth, h/settings.pxHeight
    -- else
    --   local w, h = lw.getDesktopDimensions()
    --   flags.fullscreen = true
    --   flags.borderless = true
    --   lw.setMode(w, h, flags)
    --
    --   local sx, sy = w/settings.pxWidth, h/settings.pxHeight
    --   GscaleX, GscaleY = math.min(sx, sy), math.min(sx, sy)
    -- end
    -- Signal.emit('resize')
  end
end

function love.mousepressed(x, y, button, istouch)

end

function love.resize(w, h)
  -- local settings = require 'modules.setup.settings'
  -- love.window.setMode(w, h, Gflags)
  -- local sx, sy = w/settings.pxWidth, h/settings.pxHeight
  -- GscaleX, GscaleY = math.min(sx, sy), math.min(sx, sy)
  -- Signal.emit('resize')
end
