local Brick = Class{__includes = require 'modules.entity'}
local Explosion = require 'modules.explosion'
local Rubble = require 'modules.rubble'
-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function Brick:init(x,y, quadName)
  self.pos = Vector(x,y)
  self.isBrick = true

  self.hp = 2

  self.w = 8
  self.h = 8
  self.potentialQuads = {
    parapetSingle = lg.newQuad(0, 8, 8, 8, SPRITESHEET:getDimensions()),
    flag = lg.newQuad(8, 8, 8, 8, SPRITESHEET:getDimensions()),
    parapetTiled = lg.newQuad(16, 8, 8, 8, SPRITESHEET:getDimensions()),

    wall = lg.newQuad(0, 16, 8, 8, SPRITESHEET:getDimensions()),
    wallTower = lg.newQuad(8, 16, 8, 8, SPRITESHEET:getDimensions()),
    wallWindow = lg.newQuad(16, 16, 8, 8, SPRITESHEET:getDimensions()),
    wallWindowNarrow = lg.newQuad(56, 8, 8, 8, SPRITESHEET:getDimensions()),

    wallSmallDoorLeft = lg.newQuad(64, 16, 8, 8, SPRITESHEET:getDimensions()),
    wallSmallDoorRight = lg.newQuad(72, 16, 8, 8, SPRITESHEET:getDimensions()),

    parapetVertical = lg.newQuad(104, 0, 8, 8, SPRITESHEET:getDimensions()),
    parapetFlag = lg.newQuad(104, 8, 8, 8, SPRITESHEET:getDimensions()),

    parapetCorner = lg.newQuad(104, 24, 8, 8, SPRITESHEET:getDimensions()),

  }
  local q = quadName or 'wall'
  self.quad = self.potentialQuads[q]
  self.zIndex = 4

  self.flammability = .1
  self.fireDuration = 5

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
end

-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
-- function Brick:update(dt)
--
-- end
-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function Brick:draw()
  local x, y = self.pos:unpack()
  lg.draw(SPRITESHEET, self.quad, x, y)
end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function Brick:receiveDamage(amt, src)
  self.hp = self.hp - amt
  if self.hp <= 0 then
    self:kill()
  end
end

function Brick:kill()
  local explosionChance = .1
  if love.math.random() <= explosionChance then
    table.insert(Game.entities, Explosion(self.pos.x, self.pos.y, true))
  end

  table.insert(Game.entities, Rubble(self.pos.x, self.pos.y, 'brick'))
  self:remove()
end

return Brick
