local EnemyAttack = Class{__includes = require 'modules.entity'}
-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function EnemyAttack:init(x,y, dir, type)
  self.pos = Vector(x,y)
  self.isEnemyAttack = true

  self.dir = dir
  self.type = type or 'default'

  self.w = 8
  self.h = 8
  self.zIndex = 15

  local sw, sh = SPRITESHEET:getDimensions()

  self.possibleQuads = {
    default = lg.newQuad(32, 24, 8, 8, sw, sh),
    soldier = lg.newQuad(32, 32, 8, 8, sw, sh),
    archer = lg.newQuad(32, 40, 8, 8, sw, sh),
    wizard = lg.newQuad(32, 48, 8, 8, sw, sh),
    boss = lg.newQuad(40, 48, 8, 8, sw, sh),
    impact = lg.newQuad(32, 72, 8, 8, sw, sh)
  }

  self.quad = self.possibleQuads[self.type]

  self.flammability = 0
  self.collidesWithRoy = true

  self.speed = 64
  self.damage = 1
  self.lifespan = 1

  if self.type == 'boss' then
    self.speed = 72
    self.damage = 2
    self.lifespan = 2
  end

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
  self.killTimer = Timer.after(self.lifespan, function()
    self:kill()
  end)
end


-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
function EnemyAttack:update(dt)
  local s = self.speed * dt
  local movementVector = self.dir * s
  self:move(movementVector)
end

function EnemyAttack:move(vec)
  local goalVec = self.pos + vec
  local filter = self.filter
  local actualX, actualY, cols, len = Game.world:move(self, goalVec.x, goalVec.y, filter)
  self.pos.x = actualX
  self.pos.y = actualY

  if len > 0 then
    self:handleCollision(cols, len)
  end
end

function EnemyAttack.filter(item, other)
  if other.collides == false or other.isVillager or other.isBrick or other.isBoss or other.isBat then
    return false
  else
    return 'touch'
  end
end
-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function EnemyAttack:draw()
  local x, y = self.pos:unpack()
  local r = self.dir:angleTo()
  local sx, sy = 1, 1
  local ox, oy = 4, 4
  lg.draw(SPRITESHEET, self.quad, x, y, r, sx, sy, 4, 4)

end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function EnemyAttack:handleCollision(cols, len)
  self.speed = 0
  self.collides = false
  self.quad = self.possibleQuads.impact
  for i=1, len do
    local col = cols[i]
    local other = col.other

    if other.isRoy then
      other:receiveDamage(self.damage, self)
    end
  end

  self:kill()
end

function EnemyAttack:kill()
  if self.killTimer then Timer.cancel(self.killTimer) end
  self:remove()
end

return EnemyAttack
