local Explosion = Class{__includes = require 'modules.entity'}

-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function Explosion:init(x,y, center)
  self.pos = Vector(x,y)
  self.isExplosion = true
  self.damage = 5

  if center then
    self.pos.x = self.pos.x - 8
    self.pos.y = self.pos.y - 8
  end

  self.w = 16
  self.h = 16
  self.zIndex = 11

  self.flammability = 0

  local w, h = SPRITESHEET:getDimensions()

  local g = Anim8.newGrid(16, 16, w, h, 0, 80)
  self.currentAnim = Anim8.newAnimation(g('1-8', 1), .1, 'pauseAtEnd')
  self.lifespan = 0.1 * 8

  self.damageTimer = Timer.after(0.1, function()
    self:dealDamage()
    self.damage = 0
  end)
  self.killTimer = Timer.after(.7, function()
    self:kill()
  end)

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
  TEsound.play('audio/explosion.wav', 'sfx')

end


-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
function Explosion:update(dt)
  self.currentAnim:update(dt)

end
-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function Explosion:draw()
  local x, y = self.pos:unpack()
  -- for some reason, anim8 needs the scale...
  local s = 1
  self.currentAnim:draw(SPRITESHEET, x * s, y * s, 0, s, s)
end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function Explosion:dealDamage()
  local x, y = self.pos:unpack()
  local w, h = self.w, self.h

  local filter = function(item)
    if item['receiveDamage'] then
      return true
    else
      return false
    end
  end

  local items, cols = Game.world:queryRect(x, y, w, h, filter)
  for i=1, cols do
    local item = items[i]
    item:receiveDamage(self.damage, self)
  end
end

function Explosion:kill()
  Timer.cancel(self.killTimer)
  Timer.cancel(self.damageTimer)
  self:remove()
end

return Explosion
