local Fire = Class{__includes = require 'modules.entity'}

-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function Fire:init(x,y, lifespan, target)
  self.pos = Vector(x,y)
  self.lifespan = lifespan or 10
  if target then self:attach(target) end
  self.damage = 1

  self.w = 8
  self.h = 8
  self.potentialQuads = {
    lg.newQuad(80, 8, 8, 8, SPRITESHEET:getDimensions()),
    lg.newQuad(88, 8, 8, 8, SPRITESHEET:getDimensions()),
    lg.newQuad(96, 8, 8, 8, SPRITESHEET:getDimensions()),
    lg.newQuad(80, 16, 8, 8, SPRITESHEET:getDimensions()),
    lg.newQuad(88, 16, 8, 8, SPRITESHEET:getDimensions()),
    lg.newQuad(96, 16, 8, 8, SPRITESHEET:getDimensions())
  }
  self:chooseRandomQuad()

  self.quadTimer = Timer.every(.2, function()
    self:chooseRandomQuad()
  end)

  self.expansionTimer = Timer.every(1, function()
    self:dealDamageToAttachment()
    self:decreaseLifespan()
    --self:checkForExpansion()
  end)

  self.zIndex = 8
  self.collides = false
  self.isFire = true

  self.flammability = 0

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
end

function Fire:attach(target)
  if target.attachedFire then
    target.attachedFire:remove()
  end
  target.attachedFire = self
  self.attachment = target
end
-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
function Fire:update(dt)
  if self.attachment then
    self.pos.x = self.attachment.pos.x
    self.pos.y = self.attachment.pos.y
  end
end

function Fire:chooseRandomQuad()
  local i = love.math.random(1, #self.potentialQuads)
  self.quad = self.potentialQuads[i]
end

function Fire:checkForExpansion()
  local x, y = self.pos:unpack()
  local tx, ty = x - 8, y - 8
  local tw, th = 24, 24

  local filter = function(item)
    if item.flammability == 0 or item.attachedFire or item.onFire == true then
      return false
    else
      return true
    end
  end

  local nearbyTargets, len = Game.world:queryRect(tx, ty, tw, th, filter)

  for i=1, len do
    local item = nearbyTargets[i]
    if love.math.random() <= item.flammability and item.attachedFire == nil and item.onFire ~= true then
      item.onFire = true
      item.attachedFire = self
      local ix, iy = item.pos:unpack()
      local duration = item.fireDuration or 10
      table.insert(Game.entities, Fire(ix, iy, duration, item))
    end
  end
end

function Fire:decreaseLifespan()
  self.lifespan = self.lifespan - 1
  if self.lifespan <= 0 then
    self:kill()
  end
end

-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function Fire:draw()
  local x, y = self.pos:unpack()
  lg.draw(SPRITESHEET, self.quad, x, y)

  if self.attachment and self.attachment.w == 16 and self.attachment.h == 16 then
    lg.draw(SPRITESHEET, self.quad, x + 8, y    )
    lg.draw(SPRITESHEET, self.quad, x    , y + 8)
    lg.draw(SPRITESHEET, self.quad, x + 8, y + 8)
  end
end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function Fire:dealDamageToAttachment()
  if self.attachment and self.attachment['receiveDamage'] then
    self.attachment:receiveDamage(1, self)
  end
end


function Fire:kill()
  Timer.cancel(self.quadTimer)
  Timer.cancel(self.expansionTimer)
  if self.attachment then
    self.attachment.onFire = false
    self.attachment.attachedFire = nil
  end
  self:remove()
end

return Fire
