local Fireball = Class{__includes = require 'modules.entity'}
local Fire = require 'modules.fire'
-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function Fireball:init(x,y, dir)
  self.pos = Vector(x,y)

  self.dir = dir

  self.w = 8
  self.h = 8
  self.zIndex = 4

  self.flammability = 0
  self.collidesWithRoy = false

  self.speed = 128
  self.damage = 1

  local g = Anim8.newGrid(8, 8, SPRITESHEET:getDimensions())
  self.currentAnim = Anim8.newAnimation(g('1-4', 10), .1)
  self.deathAnim = Anim8.newAnimation(g('5-11', 10), .1, 'pauseAtEnd')

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
end


-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
function Fireball:update(dt)
  self.currentAnim:update(dt)
  local s = self.speed * dt
  local movementVector = self.dir * s
  self:move(movementVector)
end

function Fireball:move(vec)
  local goalVec = self.pos + vec
  local filter = self.filter
  local actualX, actualY, cols, len = Game.world:move(self, goalVec.x, goalVec.y, filter)
  self.pos.x = actualX
  self.pos.y = actualY

  if len > 0 then
    self:handleCollision(cols, len)
  end
end

function Fireball.filter(item, other)
  if other.collides == false or other.isRoy then
    return false
  else
    return 'touch'
  end
end
-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function Fireball:draw()
  local x, y = self.pos:unpack()
  -- for some reason, anim8 needs the scale...
  local s = 1
  self.currentAnim:draw(SPRITESHEET, x * s, y * s, 0, s, s)
end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function Fireball:handleCollision(cols, len)
  self.speed = 0
  self.collides = false
  self.currentAnim = self.deathAnim
  for i=1, len do
    local col = cols[i]
    local other = col.other

    if other['receiveDamage'] then
      other:receiveDamage(self.damage, self)
    end

    if other.flammability > 0 and
    other.onFire ~= true and
    love.math.random() <= other.flammability then
      other.onFire = true

      local ix, iy = other.pos:unpack()
      local duration = other.fireDuration or 10
      local f = Fire(ix, iy, duration, other)
      other.attachedFire = f
      table.insert(Game.entities, f)

    end
  end

  Timer.after(.7, function()
    self:kill()
  end)
end

function Fireball:kill()
  self:remove()
end

return Fireball
