local PrefabList = require 'modules.prefablist'

local Brick = require 'modules.brick'
local Door = require 'modules.door'
local Villager = require 'modules.villager'
local Boss = require 'modules.boss'
local Tree = require 'modules.tree'
local Wheat = require 'modules.wheat'
local Water = require 'modules.water'
-----------------------------------------------------------
-- Helper functions
-----------------------------------------------------------
function string.explode(str, div)
  assert(type(str) == "string" and type(div) == "string", "invalid arguments")
  local o = {}
  while true do
    local pos1,pos2 = str:find(div)
    if not pos1 then
      o[#o+1] = str
      break
    end
    o[#o+1],str = str:sub(1,pos1-1),str:sub(pos2+1)
  end
  return o
end

-----------------------------------------------------------
-- Prefab Definitions
-----------------------------------------------------------
local PrefabDefs = {
  ['#'] = {constructor = Brick, args = 'wall'},
  ['>'] = {constructor = Brick, args = 'flag'},
  ['}'] = {constructor = Brick, args = 'parapetFlag'},
  ['.'] = {constructor = Brick, args = 'parapetSingle'},
  ['w'] = {constructor = Brick, args = 'parapetTiled'},
  ['H'] = {constructor = Brick, args = 'parapetVertical'},
  ['='] = {constructor = Brick, args = 'parapetCorner'},
  ['I'] = {constructor = Brick, args = 'wallTower'},
  ['O'] = {constructor = Brick, args = 'wallWindow'},
  ['o'] = {constructor = Brick, args = 'wallWindowNarrow'},
  ['D'] = {constructor = Door, args = nil},

  ['v'] = {constructor = Villager, args = nil},
  ['a'] = {constructor = Villager, args = 'archer'},
  ['s'] = {constructor = Villager, args = 'soldier'},
  ['c'] = {constructor = Villager, args = 'wizard'},
  ['d'] = {constructor = Villager, args = 'default'},

  ['@'] = {constructor = Boss, args = nil},

  ['T'] = {constructor = Tree, args = nil},
  ['f'] = {constructor = Wheat, args = nil},
  ['p'] = {constructor = Water, args = nil},
}

-----------------------------------------------------------
-- Code
-----------------------------------------------------------

local Prefab = function(x, y, which)
  local template = PrefabList[which]

  local h = select(2, template:gsub('\n', '\n'))
  local arr = string.explode(template, '\n')

  for i=1, h do
    local str = arr[i]
    for j=1, #str do
      local c = str:sub(j,j)
      if PrefabDefs[c] then
        local constructor = PrefabDefs[c].constructor
        local args = PrefabDefs[c].args
        local xx = x + (j-1) * 8
        local yy = y + (i-1) * 8
        table.insert(Game.entities, constructor(xx, yy, args))
      end
    end
  end
end

-- feels hacky:

return Prefab
