local Roy = Class{__includes = require 'modules.entity'}
local Fireball = require 'modules.fireball'
-----------------------------------------------------------
-- Init functions
-----------------------------------------------------------
function Roy:init(x,y)
  self.pos = Vector(x,y)

  self.hp = 100
  self.dead = false

  self.w = 16
  self.h = 16
  self.quad = lg.newQuad(0, 56, 16, 16, SPRITESHEET:getDimensions())
  self.zIndex = 10

  --anims
  local w, h = SPRITESHEET:getDimensions()
  local g = Anim8.newGrid(16, 16, w, h, 0, 56)
  self.anims = {}
  self.anims.idleRight = Anim8.newAnimation(g(1,1), 1, 'pauseAtEnd')
  self.anims.idleLeft = self.anims.idleRight:clone():flipH()
  self.anims.walkRight = Anim8.newAnimation(g('2-3',1), .2)
  self.anims.walkLeft = self.anims.walkRight:clone():flipH()

  self.anims.deadRight = Anim8.newAnimation(g(4,1), 1, 'pauseAtEnd')
  self.anims.deadLeft = self.anims.deadRight:clone():flipH()

  self.currentAnim = self.anims.idleRight
  self.prevFacing = 'Right'

  self.speed = 32
  self.flammability = 0
  self.isRoy = true

  self.lastDirection = Vector(0,1)

  self.canFire = true
  self.fireballCooldown = .2

  Game.world:add(self, self.pos.x, self.pos.y, self.w, self.h)
end

-----------------------------------------------------------
-- Update functions
-----------------------------------------------------------
function Roy:update(dt)
  if not self.dead then
    self:handleInput(dt)
  end
  self.currentAnim:update(dt)
end

function Roy:handleInput(dt)
  local dx, dy = 0, 0

  if lk.isDown('a', 'left') then
    dx = -1
  elseif lk.isDown('d', 'right') then
    dx = 1
  end

  if lk.isDown('w', 'up') then
    dy = -1
  elseif lk.isDown('s', 'down') then
    dy = 1
  end

  if dx ~= 0 or dy ~= 0 then
    local s = self.speed * dt
    local movementVector = Vector(dx * s, dy * s)
    self:move(movementVector)
    self.lastDirection = Vector(dx, dy)

    if dx > 0 then
      self.prevFacing = 'Right'
    elseif dx < 0 then
      self.prevFacing = 'Left'
    end

    self.currentAnim = self.anims['walk' .. self.prevFacing]
  else
    self.currentAnim = self.anims['idle' .. self.prevFacing]
  end

  if lk.isDown('space') and self.canFire then
    self:fire()
  end
end

function Roy:move(vec)
  local goalVec = self.pos + vec
  local filter = self.filter
  local actualX, actualY, cols, len = Game.world:move(self, goalVec.x, goalVec.y, filter)
  self.pos.x = actualX
  self.pos.y = actualY
end

function Roy.filter(item, other)
  if other.collides == false or other.collidesWithRoy == false then
    return false
  else
    return 'slide'
  end
end
-----------------------------------------------------------
-- Draw functions
-----------------------------------------------------------
function Roy:draw()
  local x, y = self.pos:unpack()
  --lg.draw(SPRITESHEET, self.quad, x, y)
  -- for some reason, anim8 needs the scale...
  local s = 1
  self.currentAnim:draw(SPRITESHEET, x * s, y * s, 0, s, s)
end

-----------------------------------------------------------
-- Gameplay functions
-----------------------------------------------------------
function Roy:fire()
  self.canFire = false
  Timer.after(self.fireballCooldown, function()
    self.canFire = true
  end)
  --TODO fix spawning so it launches from his mouth
  local x, y = self.pos.x, self.pos.y
  TEsound.play({'audio/royattack-2.wav', 'audio/royattack-3.wav'}, 'sfx')

  table.insert(Game.entities, Fireball(x, y, self.lastDirection))
end

function Roy:receiveDamage(amt, src)
  if not self.dead and not Game.bossIsDead then
    Game:screenShake()
    self.hp = self.hp - amt
    TEsound.play({'audio/hurt-1.wav', 'audio/hurt-2.wav', 'audio/hurt-3.wav'}, 'sfx')

    if self.hp <= 0 then
      self:kill()
    end
  end
end


function Roy:kill()
  Game:gameOver()
  self.dead = true
  self.hp = 90
  self.currentAnim = self.anims['dead' .. self.prevFacing]
  --self:remove()

end

return Roy
