local Roy = require 'modules.roy'
local Villager = require 'modules.villager'
local Cottage = require 'modules.cottage'
local Tree = require 'modules.tree'
local Water = require 'modules.water'
local Wheat = require 'modules.wheat'
local Fire = require 'modules.fire'
local Brick = require 'modules.brick'
local Door = require 'modules.door'
local Explosion = require 'modules.explosion'
local Barrier = require 'modules.barrier'
local Prefab = require 'modules.prefab'
local PrefabList = require 'modules.prefablist'
local Boss = require 'modules.boss'

-- not the best way to do it, but whatever:
local HostileTemplateList = {
  'tower', 'flagTower', 'smallFort', 'mediumFort', 'largeFort',
  'archerTower', 'wall', 'oldBuilding'
}

local EnvironmentTemplateList = {
  'grove', 'smallField', 'mediumField', 'largeField',
  'smallPond', 'mediumPond', 'largePond'
}

local World = Class{}

function World:init(w, h)

  self.w = w or 256
  self.h = h or 340

  self:addBarriers()
  self:addPlayer()

  self:addBoss()

  self:addPrefabs()
  self:addEntities()
end

function World:addBarriers()
  local maxX = self.w
  local maxY = self.h

  local barriers = {
    {0,-8,maxX,8},
    {0,maxY,maxX,8},
    {-8,-8,8,maxY + 16},
    {maxX,-8,8, maxY + 16}
  }

  for i=1, #barriers do
    local b = barriers[i]
    table.insert(Game.entities, Barrier(b))
  end
end

function World:addRandomEntities(type, count)
  local world = Game.world
  local maxX = self.w
  local maxY = self.h
  local i = 0
  local w, h = 8,8
  local filter = function(item)
    if item.collides == false then
      return false
    else
      return true
    end
  end

  if type == Cottage or type == Door then
    w, h = 16, 16
  end

  while i < count do
    local x, y = love.math.random(0, maxX - w), love.math.random(0, maxY - h)
    local items, len = world:queryRect(x, y, w, h, filter)
    if len == 0 then
      table.insert(Game.entities, type(x,y))
      i = i + 1
    end
  end
end

function World:addPlayer()
  Game.player = Roy(32,32)
  table.insert(Game.entities, Game.player)
end

function World:addPrefabs()
  self:addRandomPrefabs(25, HostileTemplateList)
  self:addRandomPrefabs(25, EnvironmentTemplateList)
end

function World:addRandomPrefabs(count, whichList)
  local tilesize = 8
  local i = 0
  local list = whichList
  local world = Game.world
  local maxX = self.w
  local maxY = self.h

  local filter = function(item)
    if item.collides == false then
      return false
    else
      return true
    end
  end

  while i < count do
    local whichTemplate = list[love.math.random(1, #list)]
    local template = PrefabList[whichTemplate]

    local h = select(2, template:gsub('\n', '\n'))
    local w = #string.explode(template, '\n')

    h = h * tilesize
    w = w * tilesize

    local x, y = love.math.random(0, maxX - w), love.math.random(0, maxY - h)
    local items, len = world:queryRect(x, y, w, h, filter)

    if len == 0 then
      Prefab(x, y, whichTemplate)
      i = i + 1
    end
  end

end

function World:addEntities()
  --add random entities
  self:addRandomEntities(Water, 20)
  self:addRandomEntities(Wheat, 20)
  self:addRandomEntities(Cottage, 50)
  self:addRandomEntities(Tree, 100)
  self:addRandomEntities(Villager, 100)
  -- uncomment for testing: (or leave it, I don't care.)
  --self:addRandomEntities(Fire, 13)
end

function World:addBoss()
  local x, y = self.w - 300, self.h - 300
  Prefab(x, y, 'wizardCastle')
end

return World
